package com.ishunwan.interactive_game_sdk;

import android.content.Context;
import android.util.Log;
import android.view.View;

import com.ishunwan.player.core.view.SWPlayerView;
import com.ishunwan.player.interactivegame.SWGameCallback;
import com.ishunwan.player.interactivegame.SWGameClient;
import com.ishunwan.player.interactivegame.SWGamePlayer;

import java.util.HashMap;
import java.util.Map;

import androidx.annotation.NonNull;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

/**
 * Created by ysh on 2020-09-10 13:54
 */
public class SWFlutterPlayerView implements PlatformView, MethodChannel.MethodCallHandler, SWGameCallback.IPlayGameListener, EventChannel.StreamHandler {
    private final static String GAME_VIEW_METHOD_CHANNEL_NAME = "com.ishunwan.interactive/game_player_";
    private final static String GAME_VIEW_EVENT_CHANNEL_NAME = "com.ishunwan.interactive/game_player_event_";

    private final static String START_PLAY_FAIL_CODE = "201";
    private final static String GRANT_CTRL_PAD_FAIL_CODE = "202";
    private final static String REVOKE_CTRL_PAD_FAIL_CODE = "203";
    private final static String GAME_BACK_EVENT_FAIL_CODE = "204";
    private final static String GAME_HOME_EVENT_FAIL_CODE = "205";

    private final static String START_PLAY_FAIL_MESSAGE = "开始游戏失败";
    private final static String GRANT_CTRL_PAD_FAIL_MESSAGE = "授予权限失败";
    private final static String REVOKE_CTRL_PAD_FAIL_MESSAGE = "收回权限失败";
    private final static String GAME_BACK_EVENT_FAIL_MESSAGE = "返回键失败";
    private final static String GAME_HOME_EVENT_FAIL_MESSAGE = "HOME键失败";

    private SWPlayerView mSWPlayerView;
    private EventChannel.EventSink mPlayViewSink;
    private SWGamePlayer mSWGamePlayer;

    public SWFlutterPlayerView(Context context, BinaryMessenger dartExecutor, int id, SWGameClient swGameClient) {
        mSWPlayerView = new SWPlayerView(context);
        mSWGamePlayer = swGameClient.createGamePlayer(mSWPlayerView);
        mSWGamePlayer.setPlayGameListener(this);
        MethodChannel channel = new MethodChannel(dartExecutor, GAME_VIEW_METHOD_CHANNEL_NAME + id);
        channel.setMethodCallHandler(this);
        EventChannel eventChannel = new EventChannel(dartExecutor, GAME_VIEW_EVENT_CHANNEL_NAME + id);
        eventChannel.setStreamHandler(this);
    }

    public void sendEvent(final String eventName, final HashMap<String, Object> map) {
        Map<String, Object> eventMap = new HashMap<>();
        eventMap.put("methodName", eventName);
        eventMap.put("args", map);
        if (mPlayViewSink != null) {
            mPlayViewSink.success(eventMap);
        }
    }

    @Override
    public View getView() {
        return mSWPlayerView;
    }

    @Override
    public void dispose() {

    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        Log.i("onMethodCall", "onMethodCall: " + call.method);
        switch (call.method) {
            case "startPlay":
                startPlay(call, result);
                break;
            case "reStart":
                reStartPlay(result);
                break;
            case "grantCtrlPadUserId":
                grantCtrlPad(call, result);
                break;
            case "revokeCtrlPadUserId":
                revokeCtrlPad(call, result);
                break;
            case "sendBackCommand":
                sendBackEvent(result);
                break;
            case "sendHomeCommand":
                sendHomeEvent(result);
                break;
            case "stopPlay":
                stopPlay(result);
                break;
            case "destroy":
                destroy(result);
                break;
        }
    }

    private void startPlay(MethodCall call, MethodChannel.Result result) {
        if (mSWGamePlayer != null) {
            try {
                mSWGamePlayer.startPlay((int) (call.argument("position")));
                result.success(null);
            } catch (Exception e) {
                resultError(result, START_PLAY_FAIL_CODE, e.getMessage());
            }
        } else {
            resultError(result, START_PLAY_FAIL_CODE, START_PLAY_FAIL_MESSAGE);
        }
    }

    private void reStartPlay(MethodChannel.Result result) {
        if (mSWGamePlayer != null) {
            mSWGamePlayer.restartPlay();
        }
        result.success(null);
    }

    private void stopPlay(MethodChannel.Result result) {
        if (mSWGamePlayer != null) {
            mSWGamePlayer.stopPlay();
        }
        result.success(null);
    }

    private void destroy(MethodChannel.Result result) {
        if (mSWGamePlayer != null) {
            mSWGamePlayer.release();
        }
        result.success(null);
    }

    private void grantCtrlPad(MethodCall call, MethodChannel.Result result) {
        if (mSWGamePlayer != null) {
            try {
                mSWGamePlayer.grantCtrl((String) (call.argument("userId")));
                result.success(null);
            } catch (Exception e) {
                resultError(result, GRANT_CTRL_PAD_FAIL_CODE, e.getMessage());
            }
        } else {
            resultError(result, GRANT_CTRL_PAD_FAIL_CODE, GRANT_CTRL_PAD_FAIL_MESSAGE);
        }
    }

    private void revokeCtrlPad(MethodCall call, MethodChannel.Result result) {
        if (mSWGamePlayer != null) {
            try {
                mSWGamePlayer.revokeCtrl((String) (call.argument("userId")));
                result.success(null);
            } catch (Exception e) {
                resultError(result, REVOKE_CTRL_PAD_FAIL_CODE, e.getMessage());
            }
        } else {
            resultError(result, REVOKE_CTRL_PAD_FAIL_CODE, REVOKE_CTRL_PAD_FAIL_MESSAGE);
        }
    }

    private void sendBackEvent(MethodChannel.Result result) {
        if (mSWGamePlayer != null) {
            try {
                mSWGamePlayer.sendGameBack();
                result.success(null);
            } catch (Exception e) {
                resultError(result, GAME_BACK_EVENT_FAIL_CODE, e.getMessage());
            }
        } else {
            resultError(result, GAME_BACK_EVENT_FAIL_CODE, GAME_BACK_EVENT_FAIL_MESSAGE);
        }
    }

    private void sendHomeEvent(MethodChannel.Result result) {
        if (mSWGamePlayer != null) {
            try {
                mSWGamePlayer.sendGameHome();
                result.success(null);
            } catch (Exception e) {
                resultError(result, GAME_HOME_EVENT_FAIL_CODE, e.getMessage());
            }
        } else {
            resultError(result, GAME_HOME_EVENT_FAIL_CODE, GAME_HOME_EVENT_FAIL_MESSAGE);
        }
    }

    private void resultError(MethodChannel.Result result, String code, String message) {
        Map<String, String> resultMap = new HashMap<>();
        resultMap.put("code", code);
        resultMap.put("message", message);
        result.success(resultMap);
    }

    @Override
    public void onPlayUICreated(SWGamePlayer swGamePlayer) {
        HashMap<String, Object> map = new HashMap<>();
        sendEvent("onPlayUICreated", map);
    }

    @Override
    public void onPlayStarted(SWGamePlayer swGamePlayer, int totalTimeSeconds) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("totalTimeSeconds", totalTimeSeconds);
        sendEvent("onPlayStarted", map);
    }

    @Override
    public void onPlayReconnectStart(SWGamePlayer swGamePlayer, int i, int i1, int i2, int i3, String s) {
    }

    @Override
    public void onPlayReconnectSuccess(SWGamePlayer swGamePlayer) {

    }

    @Override
    public void onPlayError(SWGamePlayer swGamePlayer, int status,
                            int errorFrom, int errorCode, String errorMsg) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("status", status);
        map.put("errorFrom", errorFrom);
        map.put("errorCode", errorCode);
        map.put("errorMsg", errorMsg);
        sendEvent("onPlayError", map);
    }

    @Override
    public void onPlayOrientationChanged(SWGamePlayer swGamePlayer, boolean portrait) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("portrait", portrait);
        sendEvent("onPlayOrientationChanged", map);
    }

    @Override
    public void onPlayNetWorkDelay(SWGamePlayer swGamePlayer, int delay) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("delayInMS", delay);
        sendEvent("onPlayNetWorkDelay", map);
    }

    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        mPlayViewSink = events;
    }

    @Override
    public void onCancel(Object arguments) {
        mPlayViewSink = null;
    }

    @Override
    public void onFlutterViewDetached() {

    }
}
